/*
 * Example for ccRF 3 click
 *
 * Date         Mar 2017
 * Author       Stefan Popovic
 * Copyright    GNU Public License v2
 *
 * Test configuration STM32 :
 *  MCU           :        STM32F107VCT6
 *  Dev. Board    :        EasyMx PRO v7 for STM32 ARM
 *  SW            :        mikroC PRO for ARM v5.0.0
 *
 *  NOTE          : - Place ccRF 3 click board on mikroBUS slot 1
 *                  - Connect easyTFT and turn backlight ON ( turn SW11.1 )
 *                  - Pull-down PORTD pin RD0, RD1 and RD2 and turn on Button
 *                    Press Level switch SW10 for PORTD/L (board specific)
 *                  - This example demonstrates sending and receiving packets,
 *                    with the frequency of 433 MHz of fixed size and shows
 *                    their number inside the buffer
 *                  - In order for packet to be successfuly transmitted and
 *                    received, one should be in Rx mode, while other in Tx
 *                  - Use button RD0 for initialization, RD1 for sending
 *                    and RD2 for receiving packets
 *
/*******************************************************************************
* Includes
*******************************************************************************/
#include "tft_resources.h"
#include "ccRF_3_resources.h"
#include "easyLink.h"
#include <stdbool.h>

/*******************************************************************************
* Module Preprocessor Constants
*******************************************************************************/

/*******************************************************************************
* Module Variable Definitions
*******************************************************************************/
sbit CCRF_3_RST             at GPIOC_ODR.B2;
sbit CCRF_3_CS              at GPIOD_ODR.B13;
sbit CCRF_3_GPO             at GPIOA_ODR.B4;
sbit CCRF_3_GP2             at GPIOA_ODR.B0;
sbit CCRF_3_GP3             at GPIOD_IDR.B10;

bool init_done_f            = false;
bool init_f                 = false;
bool send_f                 = false;
bool receive_f              = false;
bool reading_f              = false;
/*******************************************************************************
* Function Declarations
*******************************************************************************/
static void check_buttons( void );
static void is_initialized( void );
static void is_sending_mode( void );
static void is_receiving_mode( void );
static void system_init( void );
/*******************************************************************************
* Function Definitions
*******************************************************************************/
 /*
 * Checking Button Pressed Events
 *
 * This functions checks if one of the appointed buttons have been pressed
 */
static void check_buttons( void )
{
 if (Button(&GPIOD_IDR, 0, 100, 1))
 {
   if (!init_done_f)
  {
    init_f = true;
  }
 }
 else if (Button(&GPIOD_IDR, 1, 100, 1))
 {
  if (init_done_f)
  {
   send_f = true;
  }
 }
 else if (Button(&GPIOD_IDR, 2, 100, 1))
 {
  if (init_done_f)
  {
   receive_f = true;
  }
 }
}
 /*
 * Checking whether necessary initialization has been performed
 */
static void is_initialized( void )
{
 if ( init_f )
    {
     init_f = false;
     ccRF_Configure();
     TFT_Rectangle(15,75,300,120);
     TFT_Write_Text("Initialized.", 15, 80);
     init_done_f = 1;
    }
}
/*
 * Checking if sending mode has been requested
 */
static void is_sending_mode( void )
{
 if ( send_f )
    {
     send_f = false;
     reading_f = false;
     Delay_ms(200);
     ccRF_runTX();
     TFT_Write_Text("Sending...", 15, 130);
    }
}
/*
 * Checking if receiving mode has been requested
 */
static void is_receiving_mode( void )
{
 if ( receive_f )
    {
     receive_f = false;
     ccRF_SetRXmode();
     TFT_Write_Text("Receiving...", 15, 180);
     reading_f = true;
    }
}
/*
 * System Initialization
 *
 * Configuring MCU ports, initializing SPI module and performing reset
 */
static void system_init( void )
{
    // Configuring GPIO pins
    GPIO_Digital_Output( &GPIOD_BASE, _GPIO_PINMASK_13 );
    GPIO_Digital_Output( &GPIOC_BASE, _GPIO_PINMASK_2 );
    GPIO_Digital_Input( &GPIOA_BASE, _GPIO_PINMASK_4 );
    GPIO_Digital_Input( &GPIOA_BASE, _GPIO_PINMASK_0 );
    GPIO_Digital_Input( &GPIOD_BASE, _GPIO_PINMASK_10 );
    
    GPIO_Digital_Input(&GPIOD_IDR, _GPIO_PINMASK_0);
    GPIO_Digital_Input(&GPIOD_IDR, _GPIO_PINMASK_1);
    GPIO_Digital_Input(&GPIOD_IDR, _GPIO_PINMASK_2);

    // SPI Initialization
    SPI3_Init_Advanced( _SPI_FPCLK_DIV16, _SPI_MASTER | _SPI_8_BIT |
                    _SPI_CLK_IDLE_LOW | _SPI_FIRST_CLK_EDGE_TRANSITION |
                    _SPI_MSB_FIRST | _SPI_SS_DISABLE | _SPI_SSM_ENABLE |
                    _SPI_SSI_1, &_GPIO_MODULE_SPI3_PC10_11_12 );
    Delay_ms( 100 );

    // Perform ccRF 3 board reset
    CCRF_3_RST = 1;
    Delay_ms( 100 );
    CCRF_3_RST = 0;
    Delay_ms( 100 );
    CCRF_3_RST = 1;
    CCRF_3_CS = 1;
}

/*
 * Main function
 */
void main( void )
{
  system_init();
  display_init();
  TFT_Set_Font( &HandelGothic_BT21x22_Regular, CL_GREEN, FO_HORIZONTAL );
  TFT_Write_Text("Please  initialize  the  module.", 15, 80);

  while (1)
  {
    check_buttons();
    is_initialized();
    is_sending_mode();
    is_receiving_mode();
    if ( reading_f )
    {
     ccRF_RunRX();
    }
  }
}
/************************************************************ END OF FILE *****/